#!/bin/sh

echo Detect Large Flash Storage /etc/init.d/boottime/detectbigflash.sh > /var/tmp/dbf.log
# ---------------------------------------------------------------------------------------
# Terminal colors
NORMAL='\033[0m'
GOOD='\033[32;01m'
WARN='\033[33;01m'
BAD='\033[31;01m'
BRACKET='\033[34;01m'

# Terminal cursor positions.
# It's tempting to use absolute column positions (Esc [ n G sequences), but
# some terminal emulators can't handle them so let's use newline followed by
# relative up and right movement instead.
OKPOS='\n\033[2A\033[72C'
FAILPOS='\n\033[2A\033[71C'
INFOPOS='\n\033[A\033[4C'
# ---------------------------------------------------------------------------------------
information() {
	echo -e "${INFOPOS}${GOOD}+${NORMAL} $*"
}
# ---------------------------------------------------------------------------------------
begin() {
	echo -e " ${GOOD}*${NORMAL} $*... "
}
# ----------------------------------------------------------------------end does not exit.
end() {
	if [ "$1" = 0 ]; then
		echo -e "${OKPOS}${BRACKET}[ ${GOOD}ok${BRACKET} ]${NORMAL}"
	else
		shift
		if [ -n "$1" ]; then
			echo -e "${INFOPOS}${BAD}!${NORMAL} $*"
		fi
		echo -e "${FAILPOS}${BRACKET}[ ${BAD}fail${BRACKET} ]${NORMAL}"
	fi
}
# ---------------------------------------------------------------------------------------
format(){
  echo formattng usb drive $STORAGEAT :: $DISKAT 
	result=` mount | grep /mnt/3`
	if [ -n "$result" ];
	then
		umount /mnt/3 
	fi	
	fdisk $STORAGEAT < /usr/local/bin/sda.in 
	mke2fs $DISKAT 
	mount -t ext2 $DISKAT /mnt/3 
	cp /usr/local/bin/sda.in /mnt/3/CSTimeClockFixedStorage_DNR 
	if [ "$?" -eq "0" ];
	then
	  return 0
	else
	  return 1
	fi
}
# ---------------------------------------------------------------------------------------
set_mnt_flash_link(){
	information "remapping var/local, using /mnt/flash/varlocal"
	if [ -L /var/local ];
	then
		rm /var/local 
	fi	
	ln -s /mnt/flash/varlocal /var/local 
	chmod -R a+rwx /var/local/* 
}
# ---------------------------------------------------------------------------------------
set_usb_flash_link(){
	information "remapping var/local, using /mnt/3/local" 
	if [ -L /var/local ];
	then
		rm /var/local 
	fi	
	ln -s /mnt/3/local /var/local 
	chmod -R a+rwx /var/local/* 
}
# ---------------------------------------------------------------------------------------
copy_to_mnt_flash(){
	begin "Copying data to /mnt/flash from /mnt/3"
  # move data files back to normal folder here, and set database to rw mode.
	cp -a /mnt/3/local/* /mnt/flash/varlocal
	if [ "$?" -eq "0" ]; 
	then
		set_mnt_flash_link
		end 0
		return 0
	else
		end 1
		return 1
	fi	
}
# ---------------------------------------------------------------------------------------
copy_to_usb_flash(){
  # move data files to usb flash, and remap flash folder
	begin "Copying data to /mnt/3 from /mnt/flash"
	if [ ! -d /mnt/3/local ];
	then
		mkdir /mnt/3/local 
	fi	
	cp -a /mnt/flash/varlocal/* /mnt/3/local
	if [ "$?" -eq "0" ]; 
	then
		set_usb_flash_link
		end 0
		return 0
	else
		end 1
		return 1
	fi
}
# ---------------------------------------------------------------------------------------
copy_to_usb1_flash(){
  # move data files to other usb flash, and remap flash folder
	if [ ! -d /mnt/1/templocal ];
	then
		mkdir /mnt/1/templocal
	fi	
	cp -a /mnt/3/local/* /mnt/1/templocal 
	if [ "$?" -eq "0" ]; 
	then
		return 0
	else
    return 1
	fi
}
# ---------------------------------------------------------------------------------------
copy_from_usb1_flash(){
  # move data files to usb flash, and remap flash folder
	if [ ! -d /mnt/3/local ];
	then
		mkdir /mnt/3/local 
	fi	
	cp -a /mnt/1/templocal/* /mnt/3/local 
	if [ "$?" -eq "0" ]; 
	then
		set_usb_flash_link
		return 0
	else
    return 1
	fi
}
# ---------------------------------------------------------------------------------------
testcopy(){
	begin "Testing /mnt/flash to see if enough space"
	# first backup existing /mnt/flash capeclock db in case of failure 
	if [ -e /mnt/flash/varlocal/data/capeclock.db ];
	then
		cp /mnt/flash/varlocal/data/capeclock.db /mnt/3/oldclock.db # this might fail, but that's just too bad
	fi	
	cp /mnt/3/local/data/capeclock.db /mnt/flash/varlocal/data/capeclock.db # cp does not return an error if disk gets full
	sizeorig=$( ls -l /mnt/3/local/data/capeclock.db  | awk '{ print $5}')
	sizecopy=$( ls -l /mnt/flash/varlocal/data/capeclock.db  | awk '{ print $5}')
	if [ "$sizeorig" = "$sizecopy" ];
	then
		end 0
		return 0 # space is ok
	else
		if [ -e /mnt/flash/varlocal/data/capeclock.db ];	
		then
			rm /mnt/flash/varlocal/data/capeclock.db
		fi
		if [ -e /mnt/3/oldclock.db ];	
		then
			cp /mnt/3/oldclock.db /mnt/flash/varlocal/data/capeclock.db
		fi	
		end 1
    return 1 # space is not ok
	fi
}	
# ---------------------------------------------------------------------------------------
switch_drive() {
	# does 	using_mnt_flash exist?
  if [ -e /usr/local/bin/using_mnt_flash ];
	then
		begin "Using /mnt/flash for storage"
		set_mnt_flash_link
		end 0
		return 0 # success
	fi	

	# does using_usb_flash exist
  if [ -e /usr/local/bin/using_usb_flash ];
	then
	  if [ -e /mnt/3/local/data/update_type ];
		then
			begin "Using /mnt/3 for storage"
      set_usb_flash_link
			end 0
		  return 0 # success
	  else
		  # thinks it's using usb flash, but really it's not as there's no /mnt/3/local/data on flash disk
			begin "Data on /mnt/3 missing, switching to /mnt/flash"
			mv /usr/local/bin/using_usb_flash /usr/local/bin/using_mnt_flash 
			set_mnt_flash_link
			end 0
	  	return 0 # success
    fi	
	fi	
		
  # has the "switch to mnt flash" setting been set?
  if [ -e /usr/local/bin/switch_to_mnt_flash ];
	then
		information "Switching to /mnt/flash for storage"
		if [ -e /mnt/3/local/data/capeclock.db ];
			then
			testcopy
			if [ "$?" -eq "0" ]; 
			then
				copy_to_mnt_flash
				if [ "$?" -eq "0" ]; 
				then
					begin "Switch successful, storage on /mnt/flash used"
					mv /usr/local/bin/switch_to_mnt_flash /usr/local/bin/using_mnt_flash # switch worked
					end 0
				else
					begin "Switch unsuccessful, storage on /mnt/3 used"
					mv /usr/local/bin/switch_to_mnt_flash /usr/local/bin/using_usb_flash # switch failed
					end 0
				fi	
			else
				begin "Switch unsuccessful, storage on /mnt/3 used"
				mv /usr/local/bin/switch_to_mnt_flash /usr/local/bin/using_usb_flash # switch failed
				end 0
			fi		
			return 0
		else
			begin "No data on /mnt/3, storage on /mnt/flash used"
			mv /usr/local/bin/switch_to_mnt_flash /usr/local/bin/using_mnt_flash # switch kinda worked
			end 0
		fi		
	fi

	# has the "switch to usb" setting been set?
  if [ -e /usr/local/bin/switch_to_usb_flash ];
	then
		information "Switching to /mnt/3 for storage"
		copy_to_usb_flash
		if [ "$?" -eq "0" ]; 
		then
			begin "Switch successful, storage on /mnt/3 used"
			mv /usr/local/bin/switch_to_usb_flash /usr/local/bin/using_usb_flash # switch worked
			end 0
		else
			begin "Switch unsuccessful, storage on /mnt/flash used"
			mv /usr/local/bin/switch_to_usb_flash /usr/local/bin/using_mnt_flash # switch failed
			end 0
		fi	
		return 0
  fi
}
# ---------------------------------------------------------------------------------------	
check_for_really_old(){
	if [ ! -e /usr/local/bin/switch_to_usb_flash ];
	then 
		if [ ! -e /usr/local/bin/switch_to_mnt_flash ];
		then 
			if [ ! -e /usr/local/bin/using_mnt_flash ];
			then 
				if [ ! -e /usr/local/bin/using_usb_flash ];
				then
				  begin "creating using mnt flash"
					cp /usr/local/bin/setip.sh /usr/local/bin/using_mnt_flash
					end 0
				fi
			fi
		fi
	fi	
}
obsolete(){
	# if you get here there are no files in the /usr/local/bin to say where the data is, so assume old firmware was upgraded.
  if [ -e /mnt/3/local/data/update_type ];
	then
	  cp /usr/local/bin/setip.sh /usr/local/bin/using_usb_flash # assume using usb
		echo Data detected on usb so using that 
		set_usb_flash_link
		return 0 
	else
	  echo New usb storage found, switching to usb flash 
		copy_to_usb_flash
		if [ "$?" -eq "0" ]; 
		then
			begin "Using /mnt/3 for storage"
			cp /usr/local/bin/setip.sh /usr/local/bin/using_usb_flash  # switch worked
			end 0
		else
			begin "Using /mnt/flash for storage"
			cp /usr/local/bin/setip.sh /usr/local/bin/using_mnt_flash # switch didn't work
			end 0
		fi		
		return 0
  fi	
	return 0 
}
# ---------------------------------------------------------------------------------------
trymount (){
  begin "Data on : mount -t $1 $2 $3 /dev/sda1 /mnt/3" 
	mount -t $1 $2 $3 /dev/sda1 /mnt/3  
	if [ -e /mnt/3/CSTimeClockFixedStorage_DNR ];
	then
	  STORAGEAT="/dev/sda"
		DISKAT="/dev/sda1"
		end 0
	  return 0
	else
		end 1
		result=` mount | grep /mnt/3`
		if [ -n "$result" ];
		then
			umount /mnt/3  
		fi	
		begin "Data on : mount -t $1 $2 $3 /dev/sdb1 /mnt/3" 
		mount -t $1 $2 $3 /dev/sdb1 /mnt/3  
		if [ -e /mnt/3/CSTimeClockFixedStorage_DNR ];
		then
			STORAGEAT="/dev/sdb"
			DISKAT="/dev/sdb1"
			end 0
			return 0
		else
			end 1
		fi
	fi		
	return 1
}	
# ---------------------------------------------------------------------------------------
# no usb storage was mountable, so set clock to using existing /mnt/flash
notfound (){
	echo Large Flash Storage not found
	if [ -e /usr/local/bin/using_usb_flash ];
	then 
		mv /usr/local/bin/using_usb_flash /usr/local/bin/using_mnt_flash 
	fi
	if [ -e /usr/local/bin/switch_to_usb_flash ];
	then
		mv /usr/local/bin/switch_to_usb_flash /usr/local/bin/using_mnt_flash 
	fi
	if [ -e /usr/local/bin/switch_to_mnt_flash ];
	then
		mv /usr/local/bin/switch_to_mnt_flash /usr/local/bin/using_mnt_flash 
	fi
	if [ -e /usr/local/bin/using_mnt_flash ];
	then
		echo Using /mnt/flash 
	else
		cp /usr/local/bin/setip.sh /usr/local/bin/using_mnt_flash 
	fi
}
# ---------------------------------------------------------------------------------------
# try and change the drive over to ext2 format if on fat 32
preemptiveformat(){
  if [ -e /usr/local/bin/using_mnt_flash ];
	then
	  # using mnt/flash at the moment, so safe to reformat STORAGEAT
		information "pre-emptive format as using_mnt_flash detected"
		format
		if [ "$?" -eq "0" ];
		then 
			information "commence auto-switching to /mnt/3"
			mv /usr/local/bin/using_mnt_flash /usr/local/bin/switch_to_usb_flash 
			return 0 # drive ready for switch
		else
      information "Attempted format of usb drive failed"
			return 1 # failed, stay on mnt/flash
		fi
  fi
  if [ -e /usr/local/bin/switch_to_usb_flash ];
	then
	  # using mnt/flash at the moment, so safe to reformat STORAGEAT
		information "pre-emptive format as switch_to_usb_flash detected"
		format
		if [ "$?" -eq "0" ];
		then 
			information "commence switching to /mnt/3"
			return 0 # drive ready for switch
		else
			information "Attempted format of usb drive failed"
			mv /usr/local/bin/switch_to_usb_flash /usr/local/bin/using_mnt_flash # drive not ready for switch
			return 1 # stay on mnt/flash
		fi
  fi	
	# now it gets tricky, because data is already on flash
  if [ -e /usr/local/bin/using_usb_flash ]; 
	then
		information "pre-emptive format as using_usb_flash detected"
		if [ -e /mnt/3/local/data/capeclock.db ];
		then
			# using usb/flash at the moment, so try copy data file to mnt_flash. 
			testcopy
			if [ "$?" -eq "0" ];
			then		
				copy_to_mnt_flash
				if [ "$?" -eq "0" ]; 
				then
					mv /usr/local/bin/using_usb_flash /usr/local/bin/using_mnt_flash 
					format
					if [ "$?" -eq "0" ];
					then
						copy_to_usb_flash
						if [ "$?" -eq "0" ];
						then
							mv /usr/local/bin/using_mnt_flash /usr/local/bin/using_usb_flash 
							return 0 # success, data still on flash drive
						else
							return 1 # data ok, on mnt/flash (copy failed)
						fi	
					else
						return 1 # data ok, on mnt/flash (format failed)
					fi	
				else
					return 2 # data still on usb flash, copy to mnt failed. BUT possible /mnt/flash is now 100% full.
				fi	
			else	
				# not enough space to copy database back onto mount drive
				# work in progress, for now do nothing
				return 2 # data still on usb flash,
				
        # option of using alternate fat 32 drive in external usb port? - wip
				# check to see if usb drive is mounted on /mnt/1 
			  /usr/local/bin/mount_usb
				usbflash=`mount | grep /mnt/1`
				if [ -z "$usbflash" ];
				then
				  echo alternate flash drive is mounted
					#copy_to_usb1_flash
					#format
					#copy_from_usb1_flash
				fi	
				return 0
			fi
		else
		  # clock says it's using usb flash, but there's no data there, so safe to format
			echo pre-emptive format as using_usb_flash detected, but no data on drive 
			mv /usr/local/bin/using_usb_flash /usr/local/bin/using_mnt_flash 
			format
			if [ "$?" -eq "0" ];
			then
				mv /usr/local/bin/using_mnt_flash /usr/local/bin/switch_to_usb_flash 
				return 0 # ready to switch
			else
				return 1 # stay on mnt/flash
			fi	
		fi
  fi
	return 0
}
# ---------------------------------------------------------------------------------------
no_mnt_format(){
	# not yet tested
  return 1
	
  # look for ext2 drive. Only gets here if mount failed.
	result=`fdisk -l | grep /sda | grep Linux`
	if [ -n "$result" ];
	then
	  # drive exists
		STORAGEAT='sda'
		DISKAT='sda1'
		format
		if [ "$?" -eq "0" ];
		then
			mv /usr/local/bin/using_mnt_flash /usr/local/bin/switch_to_usb_flash 
		fi	
		return 0
	fi
}
# ---------------------------------------------------------------------------------------
# --------------------------------------------main code here--------------------------------------
# it's important to _mount_ the internal usb to /mnt/3 even if it's not being used
# otherwise it will interfere with backup / sync / restore usb functionality

/etc/init.d/clock stop
if [ -e /var/local/www/orig ];
then
	rm /var/local/www/orig 
fi	
if [ -d /var/local/www/orig ];
then
	rm /var/local/www/orig 
fi	
result=` lsmod | grep ext2`
if [ -z "$result" ];
then
	insmod /usr/local/bin/ext2.ko 
fi	
result=` mount | grep /mnt/1`
if [ -n "$result" ];
then
	umount /mnt/1
fi
result=` mount | grep /mnt/3`
if [ -n "$result" ];
then
	umount /mnt/3
fi
check_for_really_old
trymount ext2
if [ "$?" -eq "0" ]; 
then
	result=`dmesg | tail -n 1`
#	if [ "$result" = "EXT2-fs warning: mounting unchecked fs, running e2fsck is recommended" ];
#	then
	#	begin "e2fsck of mounted drive required"
	#  umount /mnt/3
	#	e2fsck -p $DISKAT
	#	trymount ext2
	#	end 0
#	fi
	switch_drive
else  
	trymount vfat -o umask=0000  
	if [ "$?" -eq "0" ]; 
	then
	  preemptiveformat # strong attempt to format usb drive to ext2 unless data on usb is too large.
		if [ "$?" -eq "0" ];
		then
			switch_drive
		fi	
	else
		notfound	
		no_mnt_format
		if [ "$?" -eq "0" ]; 
		then
			switch_drive
		fi
  fi	
fi
#information "tests"
#begin "test 1"
#end 0
#begin "test 2"
#end 1

/etc/init.d/clock start


#  -n str1 True if str1 is not null (length is greater than zero).
#  -z str1 True if str1 is null (length is zero).


# to switch from internal flash to usb flash change file
# /usr/local/bin/using_mnt_flash to 
# /usr/local/bin/switch_to_usb_flash eg
# mv /usr/local/bin/using_mnt_flash /usr/local/bin/switch_to_usb_flash
# then re boot

# to switch from usb flash to mount flash change file
# /usr/local/bin/using_usb_flash to 
# /usr/local/bin/switch_to_mnt_flash eg
# mv /usr/local/bin/using_usb_flash /usr/local/bin/switch_to_mnt_flash
# then re boot

# to see what it is currently set to do a
# ls -l /var/local
# if result shows /mnt/3/local then using usb
# if result shows /mnt/flash/varlocal then using mnt

#Superblock backups stored on blocks:
#        32768, 98304, 163840, 229376, 294912


